function train_tirbm_audio_v1(ws, num_bases, spacing, pbias, pbias_lb, pbias_lambda, epsilon, l2reg, epsdecay, nPC, sigmaPC) 

addpath ../voicebox

dataname = 'TIMIT';
batch_size = 1;

% 1. Collect the spectrograms
tic
Pall = collect_speech_data2();
toc

% 2. Run PCA
[X startframe_list] = concatenate_speech_data(Pall, randsample(length(Pall), min(length(Pall), 1000)));
% X = X-repmat(mean(X,2), 1, size(X,2)); % don't subtract mean
Cov = X*X'/size(X,2);
[V D] = eig(Cov);

% 3. Try to reconstruct the original data with PCA components
numfeat = size(X,1);
E = V(:, numfeat:-1:numfeat-nPC+1);
S = diag(subvec(diag(D), numfeat:-1:numfeat-nPC+1));
Xpc = E'*X;
Xrec = E*Xpc;

Ewhiten = diag(sqrt(diag(S)+sigmaPC).^-1)*E';
Eunwhiten = E*diag(sqrt(diag(S)+sigmaPC));

Xrec2 = Eunwhiten*Ewhiten*X;

% visualization
figure, subplot(2,1,1), imagesc(X(:, 1:500)), subplot(2,1,2), imagesc(Xrec(:, 1:500))
figure, imagesc(Xrec2(:, 1:500));
figure, imagesc(E)


% 4. Whitening (scaling)
Xw = Ewhiten*X;

% initialize the TIRBM parameters
sigma_start = 0.2;
sigma_stop = 0.2;

CD_mode = 'exp';
bias_mode = 'simple';

% Etc parameters
K_CD = 1;

% Initialization
W = [];
vbias_vec = [];
hbias_vec = [];
pars = [];

C_sigm = 1;

numchannels = size(Xw,1);


num_trials = 500;


% Initialize variables
if ~exist('pars', 'var') || isempty(pars)
    pars=[];
end

if ~isfield(pars, 'ws'), pars.ws = ws; end
if ~isfield(pars, 'num_bases'), pars.num_bases = num_bases; end
if ~isfield(pars, 'spacing'), pars.spacing = spacing; end

if ~isfield(pars, 'pbias'), pars.pbias = pbias; end
if ~isfield(pars, 'pbias_lb'), pars.pbias_lb = pbias_lb; end
if ~isfield(pars, 'pbias_lambda'), pars.pbias_lambda = pbias_lambda; end
if ~isfield(pars, 'bias_mode'), pars.bias_mode = bias_mode; end

if ~isfield(pars, 'std_gaussian'), pars.std_gaussian = sigma_start; end
if ~isfield(pars, 'sigma_start'), pars.sigma_start = sigma_start; end
if ~isfield(pars, 'sigma_stop'), pars.sigma_stop = sigma_stop; end

if ~isfield(pars, 'K_CD'), pars.K_CD = K_CD; end
if ~isfield(pars, 'CD_mode'), pars.CD_mode = CD_mode; end
if ~isfield(pars, 'C_sigm'), pars.C_sigm = C_sigm; end

if ~isfield(pars, 'num_trials'), pars.num_trials = num_trials; end
if ~isfield(pars, 'epsilon'), pars.epsilon = epsilon; end

disp(pars)

if ~exist('W', 'var') || isempty(W)
    W = 0.01*randn(pars.ws, numchannels, pars.num_bases);
end

if ~exist('vbias_vec', 'var') || isempty(vbias_vec)
    vbias_vec = zeros(numchannels,1);
end

if ~exist('hbias_vec', 'var') || isempty(hbias_vec)
    hbias_vec = -0.1*ones(pars.num_bases,1);
end

fname_prefix = sprintf('../results/audio/tirbm_audio_LB_%s_V1b_w%d_b%02d_pc%d_sigpc%g_p%g_pl%g_plambda%g_sp%d_%s_eps%g_epsdecay%g_l2reg%g_bs%02d_%s', dataname, ws, num_bases, nPC, sigmaPC, pbias, pbias_lb, pbias_lambda, spacing, CD_mode, epsilon, epsdecay, l2reg, batch_size, datestr(now, 30));
fname_save = sprintf('%s', fname_prefix);
fname_mat  = sprintf('%s.mat', fname_save);
fname_out = fname_mat;
mkdir(fileparts(fname_save));
fname_out


initialmomentum  = 0.5;
finalmomentum    = 0.9;

error_history = [];
sparsity_history = [];

dWnorm_history_CD= [];
dWnorm_history_l2= [];

Winc=0;
vbiasinc=0;
hbiasinc=0;

for t=1:num_trials
    epsilon = pars.epsilon/(1+epsdecay*t);

    tic;
    ferr_current_iter = [];
    sparsity_curr_iter = [];

    for j=1:300
        fprintf('.');
        imdata = Ewhiten*Pall{ceil(rand()*length(Pall))};
        imdata = trim_audio_for_spacing_fixconv(imdata, ws, spacing);
        imdatatr = imdata';
        imdatatr = reshape(imdatatr, [size(imdatatr,1), 1, size(imdatatr,2)]);
    
        [ferr dW dh dv poshidprobs poshidstates negdata stat]= fobj_tirbm_CD_LB_sparse_audio(imdatatr, W, hbias_vec, vbias_vec, pars, CD_mode, bias_mode, spacing, l2reg);
        ferr_current_iter = [ferr_current_iter, ferr];
        sparsity_curr_iter = [sparsity_curr_iter, mean(poshidprobs(:))];

        if t<5,
            momentum = initialmomentum;
        else
            momentum = finalmomentum;
        end

        dWnorm_history_CD(j,t) = stat.dWnorm_CD;
        dWnorm_history_l2(j,t) = stat.dWnorm_l2;

        % update parameters
        Winc = momentum*Winc + epsilon*dW;
        W = W + Winc;

        vbiasinc = momentum*vbiasinc + epsilon*dv;
        vbias_vec = vbias_vec + vbiasinc;

        hbiasinc = momentum*hbiasinc + epsilon*dh;
        hbias_vec = hbias_vec + hbiasinc;
    end
    mean_err = mean(ferr_current_iter);
    mean_sparsity = mean(sparsity_curr_iter);

    if (pars.std_gaussian > pars.sigma_stop) % stop decaying after some point
        pars.std_gaussian = pars.std_gaussian*0.99;
    end
    error_history(t) = mean(ferr_current_iter);
    sparsity_history(t) = mean(sparsity_curr_iter);

    toc

    
    fprintf('epoch %d error = %g \tsparsity_hid = %g\n', t, mean(ferr_current_iter), mean(sparsity_curr_iter));
    save(fname_mat, 'W', 'pars', 't', 'vbias_vec', 'hbias_vec', 'error_history', 'sparsity_history', 'E', 'S', 'sigmaPC', 'Ewhiten', 'Eunwhiten', 'dWnorm_history_CD', 'dWnorm_history_l2');
    
    if 0 %mod(t, 10)==0
        fname_mat_timestamp  = sprintf('%s_%04dEPOCHS.mat', fname_save, t);
        save(fname_mat_timestamp, 'W', 'pars', 't', 'vbias_vec', 'hbias_vec', 'error_history', 'sparsity_history', 'E', 'S', 'sigmaPC', 'Ewhiten', 'Eunwhiten', 'dWnorm_history_CD', 'dWnorm_history_l2');
    end
    
    disp(sprintf('results saved as %s\n', fname_mat));

end

return


function [ferr dW_total dh_total dv_total poshidprobs poshidstates negdata stat] = ...
    fobj_tirbm_CD_LB_sparse_audio(imdata, W, hbias_vec, vbias_vec, pars, CD_mode, bias_mode, spacing, l2reg)

ws = size(W,1);

poshidexp = tirbm_inference_fixconv_1d(imdata, W, hbias_vec, pars);
[poshidstates poshidprobs] = tirbm_sample_multrand_1d(poshidexp, spacing);
if strcmp(CD_mode, 'mf'), poshidstates = poshidprobs; end
    
posprods = tirbm_vishidprod_fixconv_1d(imdata, poshidprobs, ws);
poshidact = squeeze(sum(sum(poshidprobs,1),2));
posvisact = squeeze(sum(sum(imdata,1),2));


neghidstates = poshidstates;
for j=1:pars.K_CD  
    negdata = tirbm_reconstruct_LB_fixconv_1d(neghidstates, W, pars);
    neghidexp = tirbm_inference_fixconv_1d(negdata, W, hbias_vec, pars);
    [neghidstates neghidprobs] = tirbm_sample_multrand_1d(neghidexp, spacing);
    if strcmp(CD_mode, 'mf'), neghidstates = neghidprobs; end
end
negprods = tirbm_vishidprod_fixconv_1d(negdata, neghidprobs, ws);
neghidact = squeeze(sum(sum(neghidprobs,1),2));
negvisact = squeeze(sum(sum(negdata,1),2));

ferr = mean( (imdata(:)-negdata(:)).^2 );


if strcmp(bias_mode, 'none')
    dhbias = 0;
    dvbias = 0;
    dW = 0;
elseif strcmp(bias_mode, 'simple')
    dhbias = squeeze(mean(mean(poshidprobs,1),2)) - pars.pbias;
    dvbias = 0;
    dW = 0;
else 
    error('wrong adjust_bias mode!');
end

numcases1 = size(poshidprobs,1)*size(poshidprobs,2);
numcases2 = size(imdata,1)*size(imdata,2);

dW_total1 = (posprods-negprods)/numcases1;
dW_total2 = - l2reg*W;
dW_total3 = - pars.pbias_lambda*dW;
dW_total = dW_total1 + dW_total2 + dW_total3;

stat = [];
stat.dWnorm_CD = norm(vec(dW_total1));
stat.dWnorm_l2 = norm(vec(dW_total2));


dh_total = (poshidact-neghidact)/numcases1 - pars.pbias_lambda*dhbias;
dv_total = (posvisact-negvisact)/numcases2;

% fprintf('||W||=%g, ||dWprod|| = %g, ||dWl2|| = %g, ||dWsparse|| = %g\n', sqrt(sum(W(:).^2)), sqrt(sum(dW_total1(:).^2)), sqrt(sum(dW_total2(:).^2)), sqrt(sum(dW_total3(:).^2)));

return


function im2 = trim_audio_for_spacing_fixconv(im2, ws, spacing)
if mod(size(im2,2)-ws+1, spacing)~=0
    n = mod(size(im2,2)-ws+1, spacing);
    im2(:, 1:floor(n/2), :) = [];
    im2(:, end-ceil(n/2)+1:end, :) = [];
end
return


function Pall = collect_speech_data2()

Pall = {};
flist= get_speech_filenames2();
parfor i= 1:length(flist)
    if mod(i,10)==0, fprintf('.'); end
    if mod(i,1000)==0, fprintf('\n'); end
    [P, P2, y] = load_spectrogram(flist{i});
    Pall{i} = P2;
end

return


function [Pconc startframe_list] = concatenate_speech_data(Pall, idx)

count = 0;
for i=1:length(idx)
    count = count + size(Pall{idx(i)},2);
end

numfeat = size(Pall{1},1);
Pconc = zeros(numfeat, count); 
startframe_list = [];

count = 0;
for i=1:length(idx)
    startframe_list(i) = count+1;
    Pconc(:, count+1:count+size(Pall{idx(i)},2)) = Pall{idx(i)};
    count = count + size(Pall{idx(i)},2);
end

return


function flist = get_speech_filenames2()

flist = [];
fid = fopen('./flist_train.txt', 'r');
while 1
    tline = fgetl(fid);
    if ~ischar(tline), break, end
    % disp(tline);
    flist{length(flist)+1} = tline;
end
fclose(fid);

return


function [P, P2, y] = load_spectrogram(fname_prefix, opt_figure)

if ~exist('opt_figure', 'var')
    opt_figure = false;
end

fname_wav = [fname_prefix '.wav'];
fname_txt = [fname_prefix '.txt'];
fname_phn = [fname_prefix '.phn'];

% read wav file
[y,fs,ffx]=readsph(fname_wav);

% convert to spectrogram
P = []; 
P2 = get_spectrogram_orig(y, 0, fs);

return
